# lib/pokan/network.rb

module Pokan

  # Network is a helper class, used by the Pokan::Server class.
  # Its main goal is to provide a higher abstraction for networking 
  # methods provided by the Ruby core. It should not be used by
  # user code.
  class Network 

    ##
    # Sends _message_ to the given _host_ and _port_. As UDP is connectionless,
    # there's no way to be sure wheter or not the message will reach its destination.
    # UDP messages are used for communication of peers.
    def self.udp(message, host, port)
      @socket = UDPSocket.new
      bytes = @socket.send(message, 0, host, port)

      bytes > 0
    end

    ##
    # Sends _message_ to the given _host_ and _port_. TCP is a connection-oriented
    # protocol, so we need to have a server waiting for the message in the destination
    # passed as parameters.
    # If the connection cannot be established, this method returns +false+.
    # TCP messages are used for communication with seeds, since we need to be
    # sure the data really reaches its destination (for example, when someone finishes
    # the gossip server manually, pokan sends a message to a seed, telling it is about
    # to die.
    def self.tcp(message, host, port)
      begin
        @socket = TCPSocket.open(host, port)
        bytes = @socket.send(message, 0)

        bytes  > 0
      rescue
        false
      end
    end

  end
end
